// ignore_for_file: file_names

import 'dart:async';
import 'dart:convert';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:myfatoorah_flutter/MFConstants.dart';
import 'package:myfatoorah_flutter/myfatoorah_flutter.dart';

interface class ICardViewProps {
  MFCardViewStyle? cardViewStyle;
}

abstract class IMFCardView {
  Future<String> validate();
  Future<MFGetPaymentStatusResponse> pay(MFExecutePaymentRequest request,
      String lang, Function(String invoiceId) onInvoiceCreated);
}

// ignore: must_be_immutable
class MFCardPaymentView extends StatefulWidget
    implements ICardViewProps, IMFCardView {
  MFCardPaymentView({super.key, this.cardViewStyle});

  @override
  State<MFCardPaymentView> createState() => _MFCardPaymentView();

  @override
  late MFCardViewStyle? cardViewStyle;

  @override
  Future<String> validate() async {
    return await MFSDK.validate();
  }

  @override
  Future<MFGetPaymentStatusResponse> pay(MFExecutePaymentRequest request,
      String lang, Function(String invoiceId) onInvoiceCreated) async {
    return await MFSDK.pay(request, lang, onInvoiceCreated);
  }
}

class _MFCardPaymentView extends State<MFCardPaymentView> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // This is used in the platform side to register the view.
    const String viewType = MFConstants.MFCardViewModuleNAME;
    // Pass parameters to the platform side.
    Map<String, dynamic> creationParams = widget.cardViewStyle != null
        ? <String, dynamic>{
            "cardViewStyle": jsonEncode(widget.cardViewStyle?.toJson())
          }
        : <String, dynamic>{};

    switch (defaultTargetPlatform) {
      case TargetPlatform.android:
        return PlatformViewLink(
          viewType: viewType,
          surfaceFactory: (context, controller) {
            return AndroidViewSurface(
              controller: controller as AndroidViewController,
              gestureRecognizers: const <Factory<
                  OneSequenceGestureRecognizer>>{},
              hitTestBehavior: PlatformViewHitTestBehavior.opaque,
            );
          },
          onCreatePlatformView: (params) {
            return PlatformViewsService.initSurfaceAndroidView(
              id: params.id,
              viewType: viewType,
              layoutDirection: TextDirection.ltr,
              creationParams: creationParams,
              creationParamsCodec: const StandardMessageCodec(),
              onFocus: () {
                params.onFocusChanged(true);
              },
            )
              ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
              ..create();
          },
        );
      case TargetPlatform.iOS:
        return UiKitView(
          viewType: viewType,
          layoutDirection: TextDirection.ltr,
          creationParams: creationParams,
          creationParamsCodec: const StandardMessageCodec(),
        );
      default:
        throw UnsupportedError('Unsupported platform view');
    }
  }
}
