import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:myfatoorah_flutter/MFModels.dart';
import 'package:myfatoorah_flutter/MFUtils.dart';
import 'MFConstants.dart';
export 'MFCardView.dart';
export 'MFApplePay.dart';
export 'MFModels.dart';
export 'MFEnums.dart';

class _MFSDKHelperCodec extends StandardMessageCodec {
  const _MFSDKHelperCodec();

  @override
  void writeValue(WriteBuffer buffer, Object? value) {
    if (value is MFDirectPaymentRequest) {
      buffer.putUint8(BufferType.MFDirectPaymentRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else if (value is MFExecutePaymentRequest) {
      buffer.putUint8(BufferType.MFExecutePaymentRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else if (value is MFGetPaymentStatusRequest) {
      buffer.putUint8(BufferType.MFGetPaymentStatusRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else if (value is MFInitiatePaymentRequest) {
      buffer.putUint8(BufferType.MFInitiatePaymentRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else if (value is MFInitiateSessionRequest) {
      buffer.putUint8(BufferType.MFInitiateSessionRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else if (value is MFSendPaymentRequest) {
      buffer.putUint8(BufferType.MFSendPaymentRequest);
      writeValue(buffer, jsonEncode(value.toJson()));
    } else {
      super.writeValue(buffer, value);
    }
  }

  @override
  Object? readValueOfType(int type, ReadBuffer buffer) {
    switch (type) {
      case BufferType.MFDirectPaymentResponse:
        return MFDirectPaymentResponse.fromJson(
            jsonDecode(readValue(buffer)!.toString()));
      case BufferType.MFGetPaymentStatusResponse:
        return MFGetPaymentStatusResponse.fromJson(
            jsonDecode(readValue(buffer)!.toString()));
      case BufferType.MFInitiatePaymentResponse:
        return MFInitiatePaymentResponse.fromJson(
            jsonDecode(readValue(buffer)!.toString()));
      case BufferType.MFInitiateSessionResponse:
        return MFInitiateSessionResponse.fromJson(
            jsonDecode(readValue(buffer)!.toString()));
      case BufferType.MFSendPaymentResponse:
        return MFSendPaymentResponse.fromJson(
            jsonDecode(readValue(buffer)!.toString()));
      case BufferType.MFError:
        return MFError.fromJson(jsonDecode(readValue(buffer)!.toString()));
      default:
        return super.readValueOfType(type, buffer);
    }
  }
}

// ignore: non_constant_identifier_names
final MFSDK = MyFatoorahFlutter();

class MyFatoorahFlutter {
  static final MyFatoorahFlutter instance = MyFatoorahFlutter._internal();
  MyFatoorahFlutter._internal();

  factory MyFatoorahFlutter({BinaryMessenger? binaryMessenger}) {
    instance._binaryMessenger = binaryMessenger;
    return instance;
  }

  late BinaryMessenger? _binaryMessenger;

  static const MessageCodec<Object?> codec = _MFSDKHelperCodec();
  static const EventChannel eventChannel =
      EventChannel(MFConstants.MFEventChannelName);

  Future<MFInitiatePaymentResponse> initiatePayment(
      MFInitiatePaymentRequest initiatePaymentRequest, String lang) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.initiatePayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[initiatePaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFInitiatePaymentResponse>(replyList);
  }

  init(
      String apiKey, String country, String environment) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.loadConfig, codec,
        binaryMessenger: _binaryMessenger);
    channel
        .send(<Object?>[apiKey, country, environment]);
  }

  setUpActionBar(
      {String toolBarTitle = "",
      String toolBarTitleColor = "",
      String toolBarBackgroundColor = "",
      bool isShowToolBar = false}) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.setUpActionBar, codec,
        binaryMessenger: _binaryMessenger);
    channel.send(<Object?>[
      toolBarTitle,
      getColorHexFromStr(toolBarTitleColor),
      getColorHexFromStr(toolBarBackgroundColor),
      isShowToolBar
    ]);
    
  }

  Future<MFSendPaymentResponse> sendPayment(
      MFSendPaymentRequest sendPaymentRequest, String lang) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.sendPayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[sendPaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFSendPaymentResponse>(replyList);
  }

  Future<MFGetPaymentStatusResponse> getPaymentStatus(
      MFGetPaymentStatusRequest getPaymentStatusRequest, String lang) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.getPaymentStatus, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[getPaymentStatusRequest, lang]) as List<Object?>?;
    return modelParser<MFGetPaymentStatusResponse>(replyList);
  }

  Future<MFGetPaymentStatusResponse> executePayment(
      MFExecutePaymentRequest executePaymentRequest,
      String lang,
      Function(String invoiceId)? onInvoiceCreated) async {
    if (onInvoiceCreated != null) {
      eventChannel.receiveBroadcastStream().listen((invoiceId) {
        onInvoiceCreated(invoiceId);
      });
    }

    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.executePayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[executePaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFGetPaymentStatusResponse>(replyList);
  }

  Future<MFDirectPaymentResponse> executeDirectPayment(
      MFDirectPaymentRequest directPaymentRequest,
      String lang,
      Function(String invoiceId)? onInvoiceCreated) async {
    if (onInvoiceCreated != null) {
      eventChannel.receiveBroadcastStream().listen((invoiceId) {
        onInvoiceCreated(invoiceId);
      });
    }

    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.executeDirectPayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[directPaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFDirectPaymentResponse>(replyList);
  }

  Future<bool> cancelToken(String token, String lang) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.cancelToken, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList =
        await channel.send(<Object?>[token, lang]) as List<Object?>?;
    return modelParser<bool>(replyList);
  }

  Future<bool> cancelRecurringPayment(String recurringId, String lang) async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.cancelRecurringPayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList =
        await channel.send(<Object?>[recurringId, lang]) as List<Object?>?;
    return modelParser<bool>(replyList);
  }

  Future<MFInitiateSessionResponse> initiateSession(
      MFInitiateSessionRequest initiateSessionRequest,
      Function(String bin)? onCardBinChanged) async {
    if (onCardBinChanged != null) {
      eventChannel.receiveBroadcastStream().listen((bin) {
        onCardBinChanged(bin);
      });
    }

    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.initiateSession, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList =
        await channel.send(<Object?>[initiateSessionRequest]) as List<Object?>?;
    return modelParser<MFInitiateSessionResponse>(replyList);
  }

  Future<MFGetPaymentStatusResponse> pay(
      MFExecutePaymentRequest executePaymentRequest,
      String lang,
      Function(String invoiceId)? onInvoiceCreated) async {
    if (onInvoiceCreated != null) {
      eventChannel.receiveBroadcastStream().listen((invoiceId) {
        onInvoiceCreated(invoiceId);
      });
    }

    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.pay, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[executePaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFGetPaymentStatusResponse>(replyList);
  }

  Future<String> validate() async {
    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.validate, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel.send(null) as List<Object?>?;
    return modelParser<String>(replyList);
  }

  Future<MFGetPaymentStatusResponse> applePayPayment(
      MFExecutePaymentRequest executePaymentRequest,
      String lang,
      Function(String invoiceId)? onInvoiceCreated) async {
    if (onInvoiceCreated != null) {
      eventChannel.receiveBroadcastStream().listen((invoiceId) {
        onInvoiceCreated(invoiceId);
      });
    }

    final BasicMessageChannel<Object?> channel = BasicMessageChannel<Object?>(
        MFConstants.channelName.applePayPayment, codec,
        binaryMessenger: _binaryMessenger);
    final List<Object?>? replyList = await channel
        .send(<Object?>[executePaymentRequest, lang]) as List<Object?>?;
    return modelParser<MFGetPaymentStatusResponse>(replyList);
  }
}
